<?php


namespace Verifone\Hosted\Helper\Includes;
use Verifone\Hosted\Helper\Includes\VerifoneApiEntity;
use Verifone\Hosted\Helper\Includes\VerifoneHostedHelper;
use Verifone\Hosted\Helper\Settings\VerifoneHostedCartSettings;
use Exception;
use Verifone\Hosted\Model\Verifone;


class VerifoneHostedCheckout extends VerifoneApiEntity {

	/**
	 * @var string
	 */
	private $customerId;

	/**
	 * @var mixed|string
	 */
	private $currency;

	/**
	 * @var VerifoneHostedHelper
	 */
	private $helper;

	/**
	 * @var VerifoneHostedCartSettings
	 */
	private $settings;

	/**
	 * @var Verifone
	 */
	private $paymentMethod;

	/**
	 * @var
	 */
	private $quote;

	/**
	 * @var array
	 */
	protected $data;

	/**
	 * VerifoneHostedCheckout constructor.
	 *
	 * @param                                                       $quote
	 * @param VerifoneHostedCartSettings                            $settings
	 * @param \Verifone\Hosted\Helper\Includes\VerifoneHostedHelper $helper
	 * @param string                                                $customerId
	 * @param                                                       $paymentMethod
	 *
	 * @throws Exception
	 */
	public function __construct(
		$quote,
		VerifoneHostedCartSettings $settings,
		VerifoneHostedHelper $helper,
		$paymentMethod,
		$currency = '',
		$customerId = ''
	) {
		$this->quote       = $quote;
		$this->helper      = $helper;
		$this->settings    = $settings;
		$this->paymentMethod     = $paymentMethod;
		$this->currency = $currency;
		$this->customerId = $customerId;
		$this->buildData();
	}

	/**
	 * @throws Exception
	 */
	private function buildData() {
		$customer_data = [];
		if ( $this->customerId !== '' ) {
			$customer_data['customer'] =  $this->customerId;
		}

		$configurations = [
			'configurations' => [
				'card' => [
					'cvvRequired'       => $this->settings->getCvvOnPayment(),
					'captureNow'        => true,
					'paymentContractId' => $this->settings->getPaymentContractId(),
				],
			],
		];

		if ( $this->settings->is3dsEnabled() ) {
			$configurations['configurations']['card']['threedSecure'] = [
				'threeDSContractId' => $this->settings->get3dsContractId(),
				'enabled'           => $this->settings->is3dsEnabled(),
				'transactionMode'   => $this->helper->getDeviceType(),
			];
		}

		// this is by far the most dumbest way to convert to cents
		// god knows what's gonna happen with stuff like YEN
		$amount = str_replace( [ ',', '.' ], '', $this->quote->getGrandTotal() * 100  );
		$params = [
			'amount'            => $amount,
			'currencyCode'      => strtoupper( $this->currency ),
			'entityId'          => $this->settings->getEntityId(),
			// must be a string
			'merchantReference' => sprintf( '%s', $this->quote->getId() ),
			'returnUrl'         => $this->paymentMethod->getReturnUrl().'?refNo='.$this->quote->getId(),
		];

		$this->data = $params;
		$this->data = array_merge( $this->data, $customer_data );
		$this->data = array_merge( $this->data, $configurations );
	}

	/**
	 * @return array
	 */
	public function toArray() {
		return $this->data;
	}

	/**
	 * @return false|string
	 */
	public function toJson() {
		return json_encode( $this->data );
	}

}
