<?php


namespace Verifone\Hosted\Helper\Includes;
use Verifone\Hosted\Helper\Includes\VerifoneApiEntity;
use Verifone\Hosted\Helper\Settings\VerifoneHostedCartSettings;


class VerifoneHostedCustomer extends VerifoneApiEntity{

	/**
	 * @array
	 */
	private $billingAddress;

	/**
	 * @array
	 */
	private $shippingAddress;

	/**
	 * @var array
	 */
	protected $data;

	/**
	 * @var \VerifoneHostedCartSettings
	 */
	private $settings;

	/**
	 * VerifoneHostedCustomer constructor.
	 *
	 * @param                            $billingAddress
	 * @param                            $shippingAddress
	 * @param VerifoneHostedCartSettings $settings
	 */
	public function __construct(
		$billingAddress,
		$shippingAddress,
		VerifoneHostedCartSettings $settings
	) {
		$this->billingAddress = $billingAddress;
		$this->shippingAddress = $shippingAddress;
		$this->settings = $settings;
		$this->data = $this->buildData();
	}

	/**
	 * @return array
	 */
	private function buildData() {
		$data = [
			'emailAddress'=> $this->billingAddress['emailAddress'],
			'phoneNumber' => $this->billingAddress['phoneNumber'],
			'entityId'    => $this->settings->getEntityId()
		];
		if(!empty($this->billingAddress['companyName']))
		{
			$data['companyName'] = $this->billingAddress['companyName'];
		}
		$data = array_merge( $data, $this->getBillingDetails() );
		return array_merge( $data, $this->getShippingDetails() );
	}

	/**
	 * @return array[]
	 */
	private function getBillingDetails() {
		return [
			'billing' => $this->billingAddress['billing'],
		];
	}

	/**
	 * @return array[]
	 */
	private function getShippingDetails() {
		return [
			'shipping' => $this->shippingAddress,
		];
	}

	/**
	 * @return array
	 */
	public function toArray()
	{
		return $this->data;
	}

	/**
	 * @return false|string
	 */
	public function toJson() {
		return json_encode($this->data);
	}
}
