<?php

namespace Verifone\Hosted\Helper\Library;
use Exception;

class SimpleCurl {

	/**
	 * @var false|resource
	 */
	private $ch;
	private $logger;

	/**
	 * Simple_Curl constructor.
	 */
	public function __construct($logger) {
		$this->logger = $logger;

		// close any unclosed connections
		if ( is_resource( $this->ch ) ) {
			curl_close( $this->ch );
		}

		$this->ch = curl_init();
	}

	/**
	 * @param $option
	 * @param $value
	 *
	 * @return $this
	 * @throws Exception
	 */
	public function setOpt( $option, $value ) {
		// do we have an open connection?
		if(!is_resource($this->ch)) {
			$this->ch = curl_init();
		}
		if(false === curl_setopt( $this->ch, $option, $value ) ) {
			$this->logger->error('Error message', ['exception' => sprintf('There was an error when setting the curl option with name %s', $option)]);
			throw new Exception( 'There was an error in the application' );
		}

		return $this;
	}

	/**
	 * @return mixed
	 * @throws Exception
	 */
	public function getResult() {
		$result = curl_exec( $this->ch );
		$info   = curl_getinfo($this->ch );
		if ( curl_errno( $this->ch ) ) {

			$this->logger->error('Error message', ['exception' => curl_error( $this->ch )]);
			throw new Exception( 'There was an error in the application' );
		}
		curl_close( $this->ch );

		if ( ! isset( $info['http_code'] ) )  {
			$this->logger->error('Error message', ['exception' => sprintf( 'cURL failed or did not return a 200 OK status at %s line in %s file', __LINE__, __FILE__ ) ]);
			throw new Exception( 'There was an error in the application' );
		}

		switch($info['http_code'] ) {
			case 404:
			case 500:
				throw new Exception( 'There was an error in the application' );
		}

		return $result;
	}

}