<?php


namespace Verifone\Hosted\Helper\Settings;

define( 'VERIFONE_ENV_REALM', 'VerifoneUsers' );
define( 'VERIFONE_ENV_COOKIE_NAME', 'verifoneAM' );
define( 'VERIFONE_ENV_HEADER_VERSION', 'resource=2.0, protocol=1.0' );
define( 'VERIFONE_ENV_CODE_CHALLENGE_METHOD', 'S256' );
define( 'VERIFONE_ENV_CHECKOUT_EVENT_TYPE_TRANSACTION_SUCCESS', 'TRANSACTION_SUCCESS' );
define( 'VERIFONE_ENV_CHECKOUT_EVENT_TYPE_TRANSACTION_FAILED', 'TRANSACTION_FAILED' );
define( 'VERIFONE_ENV_CHECKOUT_EVENT_TYPE_TRANSACTION_DECLINED', 'TRANSACTION_DECLINED' );
define( 'VERIFONE_ENV_TRANSACTION_MODE_MOBILE_DEVICE', 'P' );
define( 'VERIFONE_ENV_TRANSACTION_MODE_COMPUTER_DEVICE', 'S' );
define( 'VERIFONE_ENV_TRANSACTION_MODE_TABLET_DEVICE', 'T' );


/**
 * Class VerifoneHostedCartSettings
 */
class VerifoneHostedCartSettings {

	private $test;
	private $region;
	private $api_username;
	private $api_password;
	private $api_client_id;
	private $test_api_username;
	private $test_api_password;
	private $test_api_client_id;
	private $_3ds_enable;
	private $payment_contract_id;
	private $_3ds_contract_id;
	private $test_entity_id;
	private $entity_id;
	private $curl_verify_ssl;
	private $test_payment_contract_id;
	private $test_3ds_contract_id;
	private $overlay_on_place_order;
	private $cvv_on_payment;

	/**
	 * @return mixed
	 */
	public function getCvvOnPayment() {
		return $this->cvv_on_payment;
	}

	/**
	 * @param $cvv_on_payment
	 *
	 * @return $this
	 */
	public function setCvvOnPayment( $cvv_on_payment ) {
		$this->cvv_on_payment = $cvv_on_payment;

		return $this;
	}

	/**
	 * @param $test_payment_contract_id
	 *
	 * @return $this
	 */
	public function setTestPaymentContractId( $test_payment_contract_id ) {
		$this->test_payment_contract_id = $test_payment_contract_id;

		return $this;
	}

	/**
	 * @param mixed $test_3ds_contract_id
	 *
	 * @return Verifone_Hosted_Cart_Settings
	 */
	public function setTest3dsContractId( $test_3ds_contract_id ) {
		$this->test_3ds_contract_id = $test_3ds_contract_id;

		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getCurlVerifySsl() {
		return $this->curl_verify_ssl;
	}

	/**
	 * @param $curl_verify_ssl
	 *
	 * @return $this
	 */
	public function setCurlVerifySsl( $curl_verify_ssl ) {
		$this->curl_verify_ssl = $curl_verify_ssl;

		return $this;
	}

	/**
	 * @param $test_entity_id
	 *
	 * @return $this
	 */
	public function setTestEntityId( $test_entity_id ) {
		$this->test_entity_id = $test_entity_id;

		return $this;
	}

	/**
	 * @return mixed
	 */
	public function getEntityId() {
		return $this->isTest() ? $this->test_entity_id : $this->entity_id;
	}

	/**
	 * @param mixed $entity_id
	 *
	 * @return VerifoneHostedCartSettings
	 */
	public function setEntityId( $entity_id ) {
		$this->entity_id = $entity_id;

		return $this;
	}

	/**
	 * @return mixed
	 */
	public function get3dsContractId() {
		return $this->isTest() ? $this->test_3ds_contract_id : $this->_3ds_contract_id;
	}

	/**
	 * @param $contractId
	 *
	 * @return $this
	 */
	public function set3dsContractId( $contractId ) {
		$this->_3ds_contract_id = $contractId;

		return $this;
	}

	/**
	 * @return mixed
	 */
	public function isTest() {
		return $this->test;
	}

	/**
	 * @return mixed
	 */
	public function getRegion() {
		return $this->region;
	}

	/**
	 * @return mixed
	 */
	public function getApiUsername() {
		return $this->isTest() ? $this->test_api_username : $this->api_username;
	}

	/**
	 * @return mixed
	 */
	public function getApiPassword() {
		return $this->isTest() ? $this->test_api_password : $this->api_password;
	}

	/**
	 * @return mixed
	 */
	public function getApiClientId() {
		return $this->isTest() ? $this->test_api_client_id : $this->api_client_id;
	}

	/**
	 * @return mixed
	 */
	public function is3dsEnabled() {
		return $this->_3ds_enable;
	}

	/**
	 * @return mixed
	 */
	public function getPaymentContractId() {
		return $this->isTest() ? $this->test_payment_contract_id : $this->payment_contract_id;
	}

	/**
	 * @param mixed $test
	 *
	 * @return VerifoneHostedCartSettings
	 */
	public function setTest( $test ) {
		$this->test = $test;

		return $this;
	}

	/**
	 * @param mixed $region
	 *
	 * @return VerifoneHostedCartSettings
	 */
	public function setRegion( $region ) {
		$this->region = $region;

		return $this;
	}

	/**
	 * @param $api_username
	 *
	 * @return $this
	 */
	public function setApiUsername( $api_username ) {
		$this->api_username = $api_username;

		return $this;
	}

	/**
	 * @param $api_password
	 *
	 * @return $this
	 */
	public function setApiPassword( $api_password ) {
		$this->api_password = $api_password;

		return $this;
	}

	/**
	 * @param $api_client_id
	 *
	 * @return $this
	 */
	public function setApiClientId( $api_client_id ) {
		$this->api_client_id = $api_client_id;

		return $this;
	}

	/**
	 * @param $test_api_username
	 *
	 * @return $this
	 */
	public function setTestApiUsername( $test_api_username ) {
		$this->test_api_username = $test_api_username;

		return $this;
	}

	/**
	 * @param $test_api_password
	 *
	 * @return $this
	 */
	public function setTestApiPassword( $test_api_password ) {
		$this->test_api_password = $test_api_password;

		return $this;
	}

	/**
	 * @param $test_api_client_id
	 *
	 * @return $this
	 */
	public function setTestApiClientId( $test_api_client_id ) {
		$this->test_api_client_id = $test_api_client_id;

		return $this;
	}

	/**
	 * @param $_3ds
	 *
	 * @return $this
	 */
	public function set3dsEnable( $_3ds ) {
		$this->_3ds_enable = $_3ds;

		return $this;
	}

	/**
	 * @param $payment_contract_id
	 *
	 * @return $this
	 */
	public function setPaymentContractId( $payment_contract_id ) {
		$this->payment_contract_id = $payment_contract_id;

		return $this;
	}

}
