<?php

namespace Verifone\Hosted\Helper;

use Verifone\Hosted\Helper\Includes\VerifoneHostedHelper;
use Verifone\Hosted\Helper\Includes\VerifoneToken;
use Verifone\Hosted\Helper\Library\EmeaVerifoneApiMapper;
use Verifone\Hosted\Helper\Library\NaVerifoneApiMapper;
use Verifone\Hosted\Helper\Library\NzauVerifoneApiMapper;
use Verifone\Hosted\Helper\Library\SimpleCurl;
use Verifone\Hosted\Helper\Settings\VerifoneHostedCartSettings;
use Verifone\Hosted\Helper\Includes\VerifoneHostedCustomer;
use Verifone\Hosted\Helper\Includes\VerifoneHostedCheckout;
use Verifone\Hosted\Helper\Includes\VerifoneHostedRefund;
use Magento\Sales\Model\Order;
use Magento\Framework\App\CacheInterface;
use Exception;


class Verifone extends \Magento\Framework\App\Helper\AbstractHelper {
	const VERIFONE_CUSTOMER = 'customer';
	const VERIFONE_CHECKOUT = 'checkout';

	protected $_session;
	protected $_logger;

	/**
	 * @var \Magento\Sales\Model\OrderFactory
	 */
	protected $_orderFactory;

	/**
	 * @var \Magento\Framework\Api\SearchCriteriaBuilder
	 */
	protected $_searchCriteriaBuilder;

	/**
	 * @var \Magento\Quote\Model\QuoteRepository
	 */
	protected $_quoteRepo;

	/**
	 * @var CacheInterface
	 */
	private $_cache;

	/**
	 * @var \Magento\Sales\Model\Service\InvoiceService
	 */
	protected $_invoiceService;

	/**
	 * @var \Magento\Framework\DB\TransactionFactory
	 */
	protected $_transactionFactory;

	private $_verifoneSettings;
	private $_simpleCurl;
	private $_mapper;

	public function __construct(
		\Magento\Framework\App\Helper\Context $context,
		\Magento\Checkout\Model\Session $session,
		\Magento\Framework\App\CacheInterface $cache,
		\Psr\Log\LoggerInterface $logger,
		\Magento\Sales\Model\OrderFactory $orderFactory,
		\Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
		\Magento\Quote\Model\QuoteRepository $quoteRepo,
		\Magento\Sales\Model\Service\InvoiceService $invoiceService,
		\Magento\Framework\DB\TransactionFactory $transactionFactory
	) {
		$this->_session               = $session;
		$this->_cache                 = $cache;
		$this->_logger                = $logger;
		$this->_orderFactory          = $orderFactory;
		$this->_searchCriteriaBuilder = $searchCriteriaBuilder;
		$this->_quoteRepo             = $quoteRepo;
		$this->_invoiceService        = $invoiceService;
		$this->_transactionFactory = $transactionFactory;
		parent::__construct( $context );

		$this->_simpleCurl = new SimpleCurl( $this->_logger );
	}

	/**
	 * @param $comment
	 *
	 * @return bool
	 * @throws \Magento\Framework\Exception\LocalizedException
	 */
	public function cancelCurrentOrder( $comment ) {
		$order = $this->_session->getLastRealOrder();
		if ( $order->getId() && $order->getState() != \Magento\Sales\Model\Order::STATE_CANCELED )
		{
			$order->registerCancellation( $comment )->save();

			return true;
		}

		return false;
	}

	/**
	 * @return bool
	 */
	public function restoreQuote() {
		return $this->_session->restoreQuote();
	}

	/**
	 * @param       $route
	 * @param array $params
	 *
	 * @return string
	 */
	public function getUrl( $route, $params = [] ) {
		return $this->_getUrl( $route, $params );
	}

	/**
	 * @param $incrementId
	 * @param $refNo
	 *
	 * @return false|\Magento\Sales\Model\Order
	 */
	public function getOrderByIncrementId( $incrementId, $refNo ) {
		$foundOrder = $this->_orderFactory->create()->loadByIncrementId( $incrementId );
		if ( $foundOrder->getId() && $refNo == $foundOrder->getExtOrderId() )
		{
			return $foundOrder;
		}

		return false;
	}

	/**
	 * @param $incrementId
	 *
	 * @return \Magento\Quote\Api\Data\CartInterface|null
	 */
	public function getQuoteByIncrementId( $incrementId ) {
		$sc       = $this->_searchCriteriaBuilder->addFilter( \Magento\Quote\Api\Data\CartInterface::KEY_RESERVED_ORDER_ID,
			$incrementId )->create();
		$quoteArr = $this->_quoteRepo->getList( $sc )->getItems();
		$quote    = null;

		if ( count( $quoteArr ) > 0 )
		{
			$quote = current( $quoteArr );
		}

		return $quote;
	}

	/**
	 * @param $paymentMethod
	 */
	public function setUpVerifoneSettings( $paymentMethod ) {
		$this->_verifoneSettings = new VerifoneHostedCartSettings();
		$this->_verifoneSettings->setTest( '1' == $paymentMethod->getConfigData( 'demo_mode' ) )
		                        ->setRegion( $paymentMethod->getConfigData( 'region' ) )
		                        ->setApiUsername( $paymentMethod->getConfigData( 'username' ) )
		                        ->setApiPassword( $paymentMethod->getConfigData( 'password' ) )
		                        ->setApiClientId( $paymentMethod->getConfigData( 'client_id' ) )
		                        ->setTestApiUsername( $paymentMethod->getConfigData( 'test_username' ) )
		                        ->setTestApiPassword( $paymentMethod->getConfigData( 'test_password' ) )
		                        ->setTestApiClientId( $paymentMethod->getConfigData( 'test_client_id' ) )
		                        ->set3dsEnable( '1' == $paymentMethod->getConfigData( 'tds' ) )
		                        ->set3dsContractId( $paymentMethod->getConfigData( 'tds_contract_id' ) )
		                        ->setPaymentContractId( $paymentMethod->getConfigData( 'payment_contract_id' ) )
		                        ->setEntityId( $paymentMethod->getConfigData( 'entity_id' ) )
		                        ->setTestEntityId( $paymentMethod->getConfigData( 'test_entity_id' ) )
		                        ->setTestPaymentContractId( $paymentMethod->getConfigData( 'test_payment_contract_id' ) )
		                        ->setTest3dsContractId( $paymentMethod->getConfigData( 'test_tds_contract_id' ) )
		                        ->setCurlVerifySsl( '1' == $paymentMethod->getConfigData( 'verifone_curl' ) )
		                        ->setCvvOnPayment( '1' == $paymentMethod->getConfigData( 'cvv_on_payment' ) );
	}

	/**
	 * @param $paymentMethod
	 */
	public function getRegionMapper( $paymentMethod ) {
		// mini factory :D
		switch ( $paymentMethod->getConfigData( 'region' ) )
		{
			case 'NZAU':
				$this->_mapper = new NzauVerifoneApiMapper( $this->_verifoneSettings );
				break;
			case 'NA':
				$this->_mapper = new NaVerifoneApiMapper( $this->_verifoneSettings );
				break;
			case 'EMEA':
				$this->_mapper = new EmeaVerifoneApiMapper( $this->_verifoneSettings );
				break;
			default:
				// will happen only on the first installation
				$this->_mapper = new NzauVerifoneApiMapper( $this->_verifoneSettings );
		}
	}

	/**
	 * @param $verifonePost
	 * @param $token
	 * @param $callType
	 *
	 * @return false|mixed
	 * @throws Exception
	 */
	private function apiCall( $verifonePost, $token, $callType ) {

		if ( $callType == self::VERIFONE_CHECKOUT )
		{
			$postCall = $this->_mapper->postCheckout();
		}
		else if ( $callType == self::VERIFONE_CUSTOMER )
		{
			$postCall = $this->_mapper->postCustomer();
		}
		else
		{
			$this->_logger->error( 'Error message', [ 'exception' => 'Function call invalid' ] );
			throw new Exception( 'An unexpected error occurred. Please retry the payment. No money has been charged.' );
		}

		$result = $this->_simpleCurl->setOpt( CURLOPT_URL, $postCall )
		                            ->setOpt( CURLOPT_RETURNTRANSFER, true )
		                            ->setOpt( CURLOPT_POST, true )
		                            ->setOpt( CURLOPT_SSL_VERIFYHOST, false )
		                            ->setOpt( CURLOPT_SSL_VERIFYPEER, false )
		                            ->setOpt( CURLOPT_POSTFIELDS, $verifonePost->toJson() )
		                            ->setOpt( CURLOPT_HTTPHEADER, [
			                            'Content-Type: application/json',
			                            'Authorization: Bearer ' . $token,
		                            ] );

		if ( $this->_verifoneSettings->getCurlVerifySsl() )
		{
			$result->setOpt( CURLOPT_SSL_VERIFYHOST, '2' )
			       ->setOpt( CURLOPT_SSL_VERIFYPEER, true );
		}
		$result = $result->getResult();

		return json_decode( $result, true );
	}

	/**
	 * @param $verifoneCustomer
	 * @param $token
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function postCustomerApiCall( $verifoneCustomer, $token ) {
		$result = $this->apiCall( $verifoneCustomer, $token, self::VERIFONE_CUSTOMER );

		if ( json_last_error() !== JSON_ERROR_NONE )
		{
			$this->_logger->error( 'Error message', [
				'exception' => sprintf( 'There was an error decoding the json response from the api response, last known error was "%s", in file %s, at line %s.  API response: %s',
					json_last_error(), __FILE__, __LINE__, $result )
			] );
			throw new Exception( 'An unexpected error occurred. Please retry the payment. No money has been charged.' );
		}

		return $result['id'];
	}

	/**
	 * @param $verifone_checkout
	 * @param $token
	 *
	 * @return array|mixed|string[]
	 * @throws Exception
	 */
	private function postCheckoutApiCall( $verifone_checkout, $token ) {
		$result = $this->apiCall( $verifone_checkout, $token, self::VERIFONE_CHECKOUT );

		if ( json_last_error() !== JSON_ERROR_NONE )
		{
			$this->_logger->error( 'Error message', [
				'exception' => sprintf( 'There was an error decoding the json response from the api response, last known error was "%s", in file %s, at line %s. API response: %s',
					json_last_error(), __FILE__, __LINE__, $result )
			] );

			return $this->failure();
		}

		if ( ! isset( $result['url'] ) )
		{
			$this->_logger->error( 'Error message', [
				'exception' => sprintf( 'The "url" key is not present in the verifone response, unable to continue, in file %s, at line %s. API response %s',
					__FILE__, __LINE__, json_encode( $result ) )
			] );

			return $this->failure();
		}

		return $result['url'];
	}

	/**
	 * @param $url
	 * @param $token
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function callApiOnResponse( $url, $token ) {
		$response = $this->_simpleCurl->setOpt( CURLOPT_URL, $url )
		                              ->setOpt( CURLOPT_RETURNTRANSFER, true )
		                              ->setOpt( CURLOPT_HTTPHEADER, [
			                              'Content-Type: application/json',
			                              'Authorization: Bearer ' . $token,
		                              ] )
		                              ->setOpt( CURLOPT_SSL_VERIFYHOST, false )
		                              ->setOpt( CURLOPT_SSL_VERIFYPEER, false );
		if ( $this->_verifoneSettings->getCurlVerifySsl() )
		{
			$response->setOpt( CURLOPT_SSL_VERIFYHOST, '2' )
			         ->setOpt( CURLOPT_SSL_VERIFYPEER, true );
		}

		try
		{
			$response = $response->getResult();
		}
		catch ( Exception $exception )
		{
			$this->_logger->critical( 'Error message', [ 'exception' => 'There was an unexpected error during the processing of your payment. Please contact the webshop for an up-to-date status.' ] );
			throw new Exception( 'An unexpected error occurred.' );
		}

		$response = json_decode( $response, true );
		if ( json_last_error() !== JSON_ERROR_NONE )
		{
			$this->_logger->critical( 'Error message', [
				'exception' => sprintf( 'There was an error decoding the json response from "%s", last known error was "%s", in file %s, at line %s. API response %s',
					$url, json_last_error(), __FILE__, __LINE__, $response )
			] );
			throw new Exception( 'An unexpected error occurred.' );
		}

		if ( ! isset( $response['events'] )
		     || ! isset( $response['merchantReference'] )
		     || ! isset( $response['transactionId'] )
		)
		{
			$this->_logger->critical( 'Error message', [
				'exception' => sprintf( 'The events or merchantReference or transactionId parameter could not be found in response, in order to proceed it must exist. API response %s',
					json_encode( $response ) )
			] );
			throw new Exception( 'An unexpected error occurred.' );
		}

		return $response;
	}

	/**
	 * @param $paymentMethod
	 * @param $params
	 *
	 * @return mixed
	 * @throws Exception
	 */
	public function checkPaymentResponse( $paymentMethod, $params ) {
		if ( ! isset( $params['checkoutId'] ) )
		{
			$this->_logger->critical( 'Error message', [
				'exception' => sprintf( 'The checkoutId parameter could not be found, in order to proceed it must exist. Current GET params: %s',
					json_encode( $params ) )
			] );
			return false;
		}

		if (!isset($params['transactionId']) || $params['transactionId'] === '' || isset($params['errorCode'])
		) {
			$this->_logger->critical( 'Error message', [
				'exception' => sprintf( 'There was an error with the payment. Current GET params: %s',
					json_encode( $params ) )
			] );
			return false;
		}

		$this->setUpVerifoneSettings( $paymentMethod );
		$this->getRegionMapper( $paymentMethod );
		$tokenHelper = new VerifoneToken( $this->_verifoneSettings, $this->_mapper, $this->_simpleCurl, $this->_logger, $this->_cache, $paymentMethod );
		$token       = $tokenHelper->getToken();
		$url         = $this->_mapper->getCheckout( $params['checkoutId'] );

		return $this->callApiOnResponse( $url, $token );
	}

	/**
	 * @param $paymentMethod
	 * @param $quote
	 *
	 * @return array
	 * @throws Exception
	 */
	public function processPayment( $paymentMethod, $quote ) {
		$this->setUpVerifoneSettings( $paymentMethod );
		$this->getRegionMapper( $paymentMethod );
		$tokenHelper     = new VerifoneToken( $this->_verifoneSettings, $this->_mapper, $this->_simpleCurl, $this->_logger, $this->_cache, $paymentMethod);
		$verifoneHelper = new VerifoneHostedHelper();

		if ( $this->_verifoneSettings->is3dsEnabled() && null === $this->_verifoneSettings->get3dsContractId() )
		{
			$this->_logger->critical( 'Error message', [ 'exception' => '3DS is enabled but no 3DS contract ID is provided in the configuration. Please fill in the field "3DS Contract Id" or disable the field "Enable/Disable 3d-secure" in the plugin settings.' ] );
			throw new Exception( 'An unexpected error occurred. Please retry the payment. No money has been charged.' );
		}

		$billingArray = $this->getBillingArray($quote->getBillingAddress());
		$shippingArray = $this->getShippingArray($quote->getShippingAddress(),$quote->getBillingAddress());

		$verifoneCustomer = new VerifoneHostedCustomer( $billingArray, $shippingArray, $this->_verifoneSettings );
		$token            = $tokenHelper->getToken();
		$customerId       = $this->postCustomerApiCall( $verifoneCustomer, $token );

		$currency = $quote->getCurrency()->getQuoteCurrencyCode();
		$verifoneCheckout = new VerifoneHostedCheckout( $quote, $this->_verifoneSettings, $verifoneHelper, $paymentMethod, $currency, $customerId );
		$checkoutUrl      = $this->postCheckoutApiCall( $verifoneCheckout, $token );

		return [
			'result'   => 'success',
			'redirect' => $checkoutUrl,
		];
	}

	public function processRefund($paymentMethod, $amount, $reason, $order, $payment){
		$this->setUpVerifoneSettings( $paymentMethod );
		$this->getRegionMapper( $paymentMethod );
		$tokenHelper     = new VerifoneToken( $this->_verifoneSettings, $this->_mapper, $this->_simpleCurl, $this->_logger, $this->_cache, $paymentMethod);
		$paymentAdditionalInfo = $payment->getAdditionalInformation(\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS);

		if(!isset($paymentAdditionalInfo['transactionId'])) {
			throw new Exception('The order was not properly complete. The required parameter Transaction ID is missing from order information. Online refund is not possible. Please contact customer support.');
		}

		$transactionId = $paymentAdditionalInfo['transactionId'];
		$token        = $tokenHelper->getToken();
		$refund_class = new VerifoneHostedRefund( $amount, $reason, $order, $transactionId);

		$response     = $this->_simpleCurl->setOpt( CURLOPT_URL, $this->_mapper->getRefund( $transactionId ) )
		                                  ->setOpt( CURLOPT_RETURNTRANSFER, true )
		                                  ->setOpt( CURLOPT_POST, true )
		                                  ->setOpt( CURLOPT_POSTFIELDS, $refund_class->toJson() )
		                                  ->setOpt( CURLOPT_SSL_VERIFYHOST, false )
		                                  ->setOpt( CURLOPT_SSL_VERIFYPEER, false )
		                                  ->setOpt( CURLOPT_HTTPHEADER, [
			                                  'Content-Type: application/json',
			                                  'Authorization: Bearer ' . $token,
		                                  ] )
		;

		if ( $this->_verifoneSettings->getCurlVerifySsl() ) {
			$response->setOpt( CURLOPT_SSL_VERIFYHOST, '2' )
			         ->setOpt( CURLOPT_SSL_VERIFYPEER, true )
			;
		}

		$response = $response->getResult();

		$response = json_decode( $response, true );
		if ( json_last_error() !== JSON_ERROR_NONE ) {
			$this->_logger->critical( 'Error message', [
				'exception' => sprintf( 'There was an error decoding the json response from the api response, last known error was "%s", in file %s, at line %s. API response: %s',
					json_last_error(), __FILE__, __LINE__, $response )
			] );
			throw new Exception( 'There was an error in the application' );
		}
		if ( ! isset( $response['id'] )
		     || ! isset( $response['status'] )
		     || 'REFUNDED' !== $response['status']
		) {
			$this->_logger->critical( 'Error message', [ 'exception' =>' Refund Error: Unable to refund transaction.' ] );
			throw new Exception('Refund Error: Unable to refund transaction.');
		}
		$payment->setAdditionalInformation(
			[ \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $response ]
		);
		$payment->save();
		$order->setState(Order::STATE_CLOSED)->setStatus($order->getConfig()->getStateDefaultStatus(Order::STATE_CLOSED));
		$order->save();
	}

	/**
	 * @param $billingAddress
	 *
	 * @return array
	 */
	public function getBillingArray($billingAddress){
		$billingMainArr = [
			'emailAddress'=> substr($billingAddress->getEmail(),0,250),
			'phoneNumber' => preg_replace('/[^0-9]+/', '', substr($billingAddress->getTelephone(), 0, 20)),
		];
		if ( ! empty( $billingAddress->getCompany() ) ) {
			$billingMainArr['companyName'] = substr($billingAddress->getCompany(),0, 100);
		}

		$billingArr = [
			'address1'    => substr($billingAddress->getStreet()[0],0,40),
		    'address2'    => (!empty($billingAddress->getStreet()[1])) ? substr($billingAddress->getStreet()[1], 0, 40) : substr($billingAddress->getStreet()[0],0,40),
		    'city'        => substr($billingAddress->getCity(),0,50),
		    'countryCode' => strtoupper($billingAddress->getCountryId() ),
		    'firstName'   => substr($billingAddress->getFirstname(),0,50),
		    'lastName'    => substr($billingAddress->getLastname(),0,50),
		    'phone'       => preg_replace('/[^0-9]+/', '', substr($billingAddress->getTelephone(), 0, 20)),
		    'postalCode'  => substr($billingAddress->getPostcode(),0,10),
		    'state'       => substr($billingAddress->getRegion(),0,10)
		];
		$billingMainArr['billing'] = $billingArr;
		return $billingMainArr;
	}

	/**
	 * @param $shippingAddress
	 *
	 * @return array
	 */
	public function getShippingArray($shippingAddress, $billingAddress){
		return [
			'address1'    => (!empty($shippingAddress->getStreet()[0])) ? substr($shippingAddress->getStreet()[0],0,40) : substr($billingAddress->getStreet()[0],0,40),
			'address2'    => (!empty($shippingAddress->getStreet()[1])) ? substr($shippingAddress->getStreet()[1],0, 40) : substr($billingAddress->getStreet()[0],0,40),
			'city'        => (!empty($shippingAddress->getCity())) ? substr($shippingAddress->getCity(),0,50) : substr($billingAddress->getCity(),0,50),
			'countryCode' => (!empty($shippingAddress->getCountryId()))  ? strtoupper($shippingAddress->getCountryId() ) : strtoupper( $billingAddress->getCountryId() ),
			'firstName'   => (!empty($shippingAddress->getFirstname()))  ? substr($shippingAddress->getFirstname(),0, 50): substr($billingAddress->getFirstname(),0, 50),
			'lastName'    => (!empty($shippingAddress->getLastname()))  ? substr($shippingAddress->getLastname(),0,50) : substr($billingAddress->getLastname(),0,50),
			'phone'       => (!empty($shippingAddress->getTelephone()))  ? preg_replace('/[^0-9]+/', '', substr($shippingAddress->getTelephone(), 0, 20)) : preg_replace('/[^0-9]+/', '', substr($billingAddress->getTelephone(), 0, 20)),
			'postalCode'  => (!empty($shippingAddress->getPostcode())) ? substr($shippingAddress->getPostcode(),0, 10) :substr($billingAddress->getPostcode(),0, 10),
			'state'       => (!empty($shippingAddress->getRegion())) ? substr($shippingAddress->getRegion(),0,50) :substr($billingAddress->getRegion(),0,50),
		];
	}

	/**
	 * @return array
	 */
	private function failure() {
		return [
			'result'   => 'failure',
			'messages' => 'There has been an error processing your order',
		];
	}

	/**
	 * @return \Magento\Sales\Model\Service\InvoiceService
	 */
	public function getInvoiceService(){
		return $this->_invoiceService;
	}

	/**
	 * @return \Magento\Framework\DB\TransactionFactory
	 */
	public function getTransactionFactory(){
		return $this->_transactionFactory;
	}
}
