<?php

namespace Verifone\Hosted\Model;

class Verifone extends \Magento\Payment\Model\Method\AbstractMethod {

	const CODE = 'verifone_hosted';

	protected $_code = self::CODE;

	protected $_isGateway = false;

	protected $_isOffline = false;

	protected $_canRefund = true;

	protected $_isInitializeNeeded = false;

	protected $_helper;

	protected $_formBlockType = \Verifone\Hosted\Block\Form\Verifone::class;

	protected $_infoBlockType = \Verifone\Hosted\Block\Info\Verifone::class;

	protected $_orderSender;

	protected $_orderRepo;

	protected $_transactionBuilder;

	protected $_taxHelper;

	protected $_locale_resolver;

	protected $_store;


	/**
	 * Verifone constructor.
	 *
	 * @param \Magento\Framework\Model\Context                                $context
	 * @param \Magento\Framework\Registry                                     $registry
	 * @param \Magento\Framework\Api\ExtensionAttributesFactory               $extensionFactory
	 * @param \Magento\Framework\Api\AttributeValueFactory                    $customAttributeFactory
	 * @param \Magento\Payment\Helper\Data                                    $paymentData
	 * @param \Magento\Framework\App\Config\ScopeConfigInterface              $scopeConfig
	 * @param \Magento\Payment\Model\Method\Logger                            $logger
	 * @param \Verifone\Hosted\Helper\Verifone                                $helper
	 * @param \Magento\Sales\Model\Order\Email\Sender\OrderSender             $orderSender
	 * @param \Magento\Sales\Api\OrderRepositoryInterface                     $orderRepo
	 * @param \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $transactionBuilder
	 * @param \Magento\Catalog\Helper\Data                                    $taxHelper
	 * @param \Magento\Framework\Locale\Resolver                              $locale_resolver
	 * @param \Magento\Store\Api\Data\StoreInterface                          $store
	 */
	public function __construct(
		\Magento\Framework\Model\Context $context,
		\Magento\Framework\Registry $registry,
		\Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory,
		\Magento\Framework\Api\AttributeValueFactory $customAttributeFactory,
		\Magento\Payment\Helper\Data $paymentData,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Payment\Model\Method\Logger $logger,
		\Verifone\Hosted\Helper\Verifone $helper,
		\Magento\Sales\Model\Order\Email\Sender\OrderSender $orderSender,
		\Magento\Sales\Api\OrderRepositoryInterface $orderRepo,
		\Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $transactionBuilder,
		\Magento\Catalog\Helper\Data $taxHelper,
		\Magento\Framework\Locale\Resolver $locale_resolver,
		\Magento\Store\Api\Data\StoreInterface $store
	) {
		$this->_helper             = $helper;
		$this->_orderSender        = $orderSender;
		$this->_orderRepo          = $orderRepo;
		$this->_transactionBuilder = $transactionBuilder;
		$this->_taxHelper          = $taxHelper;
		$this->_locale_resolver    = $locale_resolver;
		$this->_store              = $store;

		parent::__construct(
			$context,
			$registry,
			$extensionFactory,
			$customAttributeFactory,
			$paymentData,
			$scopeConfig,
			$logger
		);
	}

	/**
	 * @param $response
	 */
	protected function _createTransaction( $response, $transactionFactory, $order, $invoice = null ) {
		$transaction = null;
		if ( $invoice )
		{
			$transaction = $transactionFactory->create()
			                                  ->addObject( $invoice )
			                                  ->addObject( $order );

			$transaction->save();
		}
		$order->save();
	}

	/**
	 * @param \Magento\Sales\Model\Order $order
	 * @param                            $response
	 */
	public function postProcessing( \Magento\Sales\Model\Order $order, $response, $invoiceService, $transactionFactory ) {
		try
		{

			if ( $this->getSendInvoice() == '1' && $order->canInvoice())
			{
				$payment = $order->getPayment();
				$payment->setTransactionId($response['transactionId']);
				$payment->setAdditionalInformation( 'verifone_order_status', 'approved' );
				$payment->setAdditionalInformation(
					'verifone_order_number',
					$response['transactionId']
				);
				$payment->setAdditionalInformation(
					[ \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $response ]
				);
				$payment->setShouldCloseParentTransaction( true );
				$payment->setIsTransactionClosed( 0 );
				$payment->registerCaptureNotification( $order->getGrandTotal(), true );
				$payment->setParentTransactionId( null );
				$payment->save();
				$order->save();

				$invoice = $payment->getCreatedInvoice();
				if ( ! $invoice )
				{
					$invoice = $invoiceService->prepareInvoice( $order );
					$invoice->setRequestedCaptureCase( \Magento\Sales\Model\Order\Invoice::CAPTURE_ONLINE );
					$invoice->register();
				}

				// notify customer
				if ( $invoice )
				{
					$this->_createTransaction( $response, $transactionFactory, $order, $invoice );
					if ( ! $order->getEmailSent() )
					{
						$this->_orderSender->send( $order );
						$order->addCommentToStatusHistory(
							__( 'Notified customer about invoice #%1.', $invoice->getIncrementId(), false )
						)->setIsCustomerNotified(
							true
						)->save();
					}
				}
				$order->setStatus( $this->getOrderStatus() );
				$order->save();
				$this->_orderRepo->save( $order );

				return $order;
			}else{
				$payment = $order->getPayment();
				$payment->setTransactionId($response['transactionId']);
				$payment->setIsTransactionClosed(0);
				$payment->setAdditionalInformation( 'verifone_order_status', 'approved' );
				$payment->setAdditionalInformation(
					[ \Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $response ]
				);
				$payment->setShouldCloseParentTransaction( true );
				$payment->setParentTransactionId( null );
				$payment->save();

				//Only create the transaction here & only if no invoice creation trigger is set in Magento2 admin, connectors settings
				$formattedPrice = $order->getBaseCurrency()->formatTxt(
					$order->getGrandTotal()
				);

				$message        = __( 'The authorized amount is %1.', $formattedPrice );
				$trans       = $this->_transactionBuilder;
				$transaction = $trans->setPayment( $payment )
				                     ->setOrder( $order )
				                     ->setTransactionId( $response['transactionId'] )
				                     ->setFailSafe( true )
				                     ->build( \Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE );

				$payment->addTransactionCommentsToOrder($transaction, $message);
				$payment->save();
				$order->setStatus( $this->getOrderStatus() );
				$order->save();
				$this->_orderRepo->save( $order );

				return $order;
			}
		}
		catch ( Exception $e )
		{
			throw new Exception( sprintf( 'Error Creating Invoice: "%s"', $e->getMessage() ) );
		}
	}

	/**
	 * @return string
	 */
	public function getRedirectUrl() {
		return $this->_helper->getUrl( $this->getConfigData( 'redirect_url' ) );
	}

	/**
	 * @return string
	 */
	public function getReturnUrl() {
		return $this->_helper->getUrl( $this->getConfigData( 'return_url' ) );
	}

	/**
	 * @return string
	 */
	public function getCancelUrl() {
		return $this->_helper->getUrl( $this->getConfigData( 'cancel_url' ) );
	}

	/**
	 * @return mixed
	 */
	public function getOrderStatus() {
		return $this->getConfigData( 'order_status' );
	}

	/**
	 * @return mixed
	 */
	public function getSendInvoice() {
		return $this->getConfigData( 'send_invoice' );
	}

	public function refund(
		\Magento\Payment\Model\InfoInterface $payment,
		$amount
	) {

		if ($amount <= 0) {
			throw new \Magento\Framework\Exception\LocalizedException(__('Invalid amount for refund.'));
		}

		if(!$this->_canRefund){
			throw new \Magento\Framework\Exception\LocalizedException(__('The refund action is not available.'));
		}

		if (!$payment->getParentTransactionId()) {
			throw new \Magento\Framework\Exception\LocalizedException(__('Invalid transaction ID.'));
		}

		$reason = (isset($_POST['creditmemo']['comment_text'])) ? $_POST['creditmemo']['comment_text'] : '';
		$order = $payment->getOrder();
		$paymentMethod = $payment->getMethodInstance();
		$this->_helper->processRefund($paymentMethod, $amount, $reason, $order, $payment);
		return $this;
	}


}
