<?php
namespace Verifone\Hosted\Test\Unit;
use Magento\Quote\Model\Quote;
use Magento\Sales\Model\Order;
use Verifone\Hosted\Helper\Includes\MobileDetect;
use Verifone\Hosted\Helper\Includes\VerifoneHostedCheckout;
use Verifone\Hosted\Helper\Includes\VerifoneHostedCustomer;
use Verifone\Hosted\Helper\Includes\VerifoneHostedHelper;
use Verifone\Hosted\Helper\Includes\VerifoneHostedRefund;
use Verifone\Hosted\Helper\Library\EmeaVerifoneApiMapper;
use Verifone\Hosted\Helper\Library\NaVerifoneApiMapper;
use Verifone\Hosted\Helper\Library\NzauVerifoneApiMapper;
use Verifone\Hosted\Helper\Library\SimpleCurl;
use Verifone\Hosted\Helper\Settings\VerifoneHostedCartSettings;
use Verifone\Hosted\Model\Verifone;
use Psr\Log\LoggerInterface;
use Exception;

class TestingClass extends \PHPUnit\Framework\TestCase
{
	private $simpleCurl;


	public function testTest()
	{
		$this->assertTrue(true);
	}

	public function test_CheckoutArray(){

		$quoteMock =  $this->getMockBuilder( Quote::class )
		                   ->disableOriginalConstructor()
		                   ->setMethods( [ 'getQuoteCurrencyCode', 'getCurrency', 'getGrandTotal' ] )
		                   ->getMock();

		$settings = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                ->disableOriginalConstructor()
		                ->setMethods( [ 'getPaymentContractId', 'get3dsContractId', 'is3dsEnabled', 'getEntityId' ] )
		                ->getMock();

		$helper = $this->getMockBuilder( VerifoneHostedHelper::class )
		                 ->disableOriginalConstructor()
		                 ->setMethods( [ 'getDeviceType', 'setMobileDetectClass' ] )
		                 ->getMock();

		$paymentMethod = $this->getMockBuilder( Verifone::class )
		                      ->disableOriginalConstructor()
		                      ->setMethods( [ 'getReturnUrl'] )
		                      ->getMock();

		$customerId = 1;
		$currency = 'USD';

		$verifoneHostedCheckout = new VerifoneHostedCheckout( $quoteMock, $settings, $helper, $paymentMethod, $currency, $customerId);
		$data = [
			'amount'            => "0",
			'currencyCode'      => "USD",
			'entityId'          => null,
			'merchantReference' => "",
			'returnUrl'         => "?refNo=",
			'customer'          => 1,
			'configurations'    => [
				'card' => [
					'cvvRequired'       => null,
					'captureNow'        => true,
					'paymentContractId' => null,
				],
			]
		];
		$this->assertEquals( $data, $verifoneHostedCheckout->toArray() );
		$this->assertEquals( strlen( json_encode( $data ) ), strlen( $verifoneHostedCheckout->toJson() ) );
	}

	public function test_CustomerArray(){

		$subBillingArray = [
			'address1'    => 'billing_test_addr_1',
			'address2'    => 'billing_test_addr_2',
			'city'        => 'billing_test_city',
			'countryCode' => 'billing_TEST_COUNTRY',
			'firstName'   => 'billing_test_first_name',
			'lastName'    => 'billing_test_last_name',
			'phone'       => 'billing_test_phone',
			'postalCode'  => 'billing_test_post_code',
			'state'       => 'billing_test_state'
		];

		$billingArray = [
			'companyName' => 'test_company',
			'emailAddress'=> 'test_email',
			'phoneNumber' => 'test_phone_number',
			'billing' => $subBillingArray
		];

		$shippingArray =[
			'address1'    => 'shipping_test_addr_1',
			'address2'    => 'shipping_test_addr_2',
			'city'        => 'shipping_test_city',
			'countryCode' => 'shipping_TEST_COUNTRY',
			'firstName'   => 'shipping_test_first_name',
			'lastName'    => 'shipping_test_last_name',
			'phone'       => 'shipping_test_phone',
			'postalCode'  => 'shipping_test_post_code',
			'state'       => 'shipping_test_state'
		];

		$data = [
			'emailAddress'=> 'test_email',
			'phoneNumber' => 'test_phone_number',
			'entityId'=> null,
			'companyName'=> 'test_company',
			'billing' => $subBillingArray,
			'shipping' => $shippingArray
		];

		$settings = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                 ->disableOriginalConstructor()
		                 ->setMethods( [ 'getPaymentContractId', 'get3dsContractId', 'is3dsEnabled', 'getEntityId' ] )
		                 ->getMock();


		$verifoneHostedCustomer = new VerifoneHostedCustomer($billingArray, $shippingArray, $settings );

		$this->assertEquals($data, $verifoneHostedCustomer->toArray());
		$this->assertEquals(strlen(json_encode($data)), strlen($verifoneHostedCustomer->toJson()));
	}

	public function test_getDeviceType(){
		$mobileDetectMock = $this->getMockBuilder( MobileDetect::class )
		                         ->disableOriginalConstructor()
		                         ->setMethods( [
			                         'isMobile',
			                         'isTablet',
		                         ] )
		                         ->getMock()
		;

		$mobileDetectMock->expects( $this->any() )
		                 ->method( 'isMobile' )
		                 ->willReturn( false )
		;
		$mobileDetectMock->expects( $this->any() )
		                 ->method( 'isTablet' )
		                 ->willReturn( false )
		;

		$testClass = new VerifoneHostedHelper();
		$testClass->setMobileDetectClass($mobileDetectMock);

		$this->assertEquals('S', $testClass->getDeviceType());

		// mobile
		$mobileDetectMock = $this->getMockBuilder( MobileDetect::class )
		                         ->disableOriginalConstructor()
		                         ->setMethods( [
			                         'isMobile',
			                         'isTablet',
		                         ] )
		                         ->getMock()
		;

		$mobileDetectMock->expects( $this->any() )
		                 ->method( 'isMobile' )
		                 ->willReturn( true )
		;
		$mobileDetectMock->expects( $this->any() )
		                 ->method( 'isTablet' )
		                 ->willReturn( false )
		;

		$testClass = new VerifoneHostedHelper();
		$testClass->setMobileDetectClass($mobileDetectMock);

		$this->assertEquals('P', $testClass->getDeviceType());

		//tablet
		$mobileDetectMock = $this->getMockBuilder( MobileDetect::class )
		                         ->disableOriginalConstructor()
		                         ->setMethods( [
			                         'isMobile',
			                         'isTablet',
		                         ] )
		                         ->getMock()
		;

		$mobileDetectMock->expects( $this->any() )
		                 ->method( 'isMobile' )
		                 ->willReturn( false )
		;
		$mobileDetectMock->expects( $this->any() )
		                 ->method( 'isTablet' )
		                 ->willReturn( true )
		;

		$testClass = new VerifoneHostedHelper();
		$testClass->setMobileDetectClass($mobileDetectMock);

		$this->assertEquals('T', $testClass->getDeviceType());
	}


	public function setUp(): void {
		$loggerMock =  $this->getMockBuilder( LoggerInterface::class )
		                   ->disableOriginalConstructor()
		                   ->getMock();

		$this->simpleCurl = new SimpleCurl($loggerMock);
		parent::setUp();
	}

	public function test_ExpectRegularException()
	{
		$this->expectException( Exception::class );
		$this->simpleCurl->getResult();
	}

	public function test_setOpt()
	{
		$curl = $this->simpleCurl->setOpt(CURLOPT_BINARYTRANSFER, true);
		$this->assertInstanceOf(SimpleCurl::class, $curl);
	}


	public function test_EmeaSettersGetters()
	{
		$settingsMock = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                     ->disableOriginalConstructor()
		                     ->setMethods( [ 'isTest' ] )
		                     ->getMock()
		;

		$settingsMock->expects($this->any())
		             ->method('isTest')
		             ->willReturn(true);
		$emeaMapper = new EmeaVerifoneApiMapper($settingsMock);

		$this->assertSame('https://cst1.test-vam.vfims.com', $emeaMapper->getToken());
		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout/example_id',
			$emeaMapper->getCheckout('example_id')
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer/1',
			$emeaMapper->getCustomer(1)
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/ds-entity-service/paymentContracts/1',
			$emeaMapper->getPaymentContract(1)
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/api/v1/transactions/1/refund',
			$emeaMapper->getRefund(1)
		);

		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout',
			$emeaMapper->postCheckout()
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer',
			$emeaMapper->postCustomer()
		);

		//----
		// This basically does nothing atm since we don't have an actual test api available
		$settingsMock = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                     ->disableOriginalConstructor()
		                     ->setMethods( [ 'isTest' ] )
		                     ->getMock()
		;

		$settingsMock->expects($this->any())
		             ->method('isTest')
		             ->willReturn(false);
		$emeaMapper = new EmeaVerifoneApiMapper($settingsMock);

		$this->assertSame('https://cst1.test-vam.vfims.com', $emeaMapper->getToken());
		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout/example_id',
			$emeaMapper->getCheckout('example_id')
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer/1',
			$emeaMapper->getCustomer(1)
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/ds-entity-service/paymentContracts/1',
			$emeaMapper->getPaymentContract(1)
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/api/v1/transactions/1/refund',
			$emeaMapper->getRefund(1)
		);

		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout',
			$emeaMapper->postCheckout()
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer',
			$emeaMapper->postCustomer()
		);
	}


	public function test_NaSettersGetters()
	{
		$settingsMock = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                     ->disableOriginalConstructor()
		                     ->setMethods( [ 'isTest' ] )
		                     ->getMock()
		;

		$settingsMock->expects($this->any())
		             ->method('isTest')
		             ->willReturn(true);
		$naMapper = new NaVerifoneApiMapper($settingsMock);

		$this->assertSame('https://cst1.test-vam.vfims.com', $naMapper->getToken());
		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout/example_id',
			$naMapper->getCheckout('example_id')
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer/1',
			$naMapper->getCustomer(1)
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/ds-entity-service/paymentContracts/1',
			$naMapper->getPaymentContract(1)
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/api/v1/transactions/1/refund',
			$naMapper->getRefund(1)
		);

		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout',
			$naMapper->postCheckout()
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer',
			$naMapper->postCustomer()
		);

		//----
		// This basically does nothing atm since we don't have an actual test api available
		$settingsMock = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                     ->disableOriginalConstructor()
		                     ->setMethods( [ 'isTest' ] )
		                     ->getMock()
		;

		$settingsMock->expects($this->any())
		             ->method('isTest')
		             ->willReturn(false);
		$naMapper = new NaVerifoneApiMapper($settingsMock);

		$this->assertSame('https://cst1.test-vam.vfims.com', $naMapper->getToken());
		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout/example_id',
			$naMapper->getCheckout('example_id')
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer/1',
			$naMapper->getCustomer(1)
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/ds-entity-service/paymentContracts/1',
			$naMapper->getPaymentContract(1)
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/api/v1/transactions/1/refund',
			$naMapper->getRefund(1)
		);

		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout',
			$naMapper->postCheckout()
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer',
			$naMapper->postCustomer()
		);
	}

	public function test_NzauSettersGetters()
	{
		$settingsMock = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                     ->disableOriginalConstructor()
		                     ->setMethods( [ 'isTest' ] )
		                     ->getMock()
		;

		$settingsMock->expects($this->any())
		             ->method('isTest')
		             ->willReturn(true);
		$nzauMapper = new NzauVerifoneApiMapper($settingsMock);

		$this->assertSame('https://cst1.test-vam.vfims.com',$nzauMapper->getToken());
		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout/example_id',
			$nzauMapper->getCheckout('example_id')
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer/1',
			$nzauMapper->getCustomer(1)
		);
		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/ds-entity-service/paymentContracts/1',
			$nzauMapper->getPaymentContract(1)
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/api/v1/transactions/1/refund',
			$nzauMapper->getRefund(1)
		);

		$this->assertSame('https://checkout_cst.dimebox.com/v1/checkout',
			$nzauMapper->postCheckout()
		);

		$this->assertSame('https://cst2.test-gsc.vfims.com/oidc/customer-service/v1/customer',
			$nzauMapper->postCustomer()
		);

		//----
		// This basically does nothing atm since we don't have an actual test api available
		$settingsMock = $this->getMockBuilder( VerifoneHostedCartSettings::class )
		                     ->disableOriginalConstructor()
		                     ->setMethods( [ 'isTest' ] )
		                     ->getMock()
		;

		$settingsMock->expects($this->any())
		             ->method('isTest')
		             ->willReturn(false);
		$nzauMapper = new NzauVerifoneApiMapper($settingsMock);

		$this->assertSame('https://nz.vam.verifone.cloud', $nzauMapper->getToken());
		$this->assertSame('https://checkout_nz.dimebox.com/v1/checkout/example_id',
			$nzauMapper->getCheckout('example_id')
		);
		$this->assertSame('https://nz.gsc.verifone.cloud/oidc/customer-service/v1/customer/1',
			$nzauMapper->getCustomer(1)
		);
		$this->assertSame('https://nz.gsc.verifone.cloud/oidc/ds-entity-service/paymentContracts/1',
			$nzauMapper->getPaymentContract(1)
		);

		$this->assertSame('https://nz.gsc.verifone.cloud/oidc/api/v1/transactions/1/refund',
			$nzauMapper->getRefund(1)
		);

		$this->assertSame('https://checkout_nz.dimebox.com/v1/checkout',
			$nzauMapper->postCheckout()
		);

		$this->assertSame('https://nz.gsc.verifone.cloud/oidc/customer-service/v1/customer',
			$nzauMapper->postCustomer()
		);
	}

	public function test_SettingsSettersGetters()
	{
		$settings = new VerifoneHostedCartSettings();

		// non conditional
		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setCurlVerifySsl(true));
		$this->assertSame(true, $settings->getCurlVerifySsl());
		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setRegion('NA'));
		$this->assertSame('NA', $settings->getRegion());
		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->set3dsEnable(true));
		$this->assertSame(true, $settings->is3dsEnabled());

		// conditional
		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTest(true));
		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTestPaymentContractId('test_true'));
		$this->assertSame('test_true', $settings->getPaymentContractId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTest3dsContractId('test_true'));
		$this->assertSame('test_true', $settings->get3dsContractId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTestEntityId('test_true'));
		$this->assertSame('test_true', $settings->getEntityId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->set3dsContractId('test_true'));
		$this->assertSame('test_true', $settings->get3dsContractId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTestEntityId('test_true'));
		$this->assertSame('test_true', $settings->getEntityId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTestApiUsername('test_true'));
		$this->assertSame('test_true', $settings->getApiUsername());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTestApiPassword('test_true'));
		$this->assertSame('test_true', $settings->getApiPassword());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setTestApiClientId('test_true'));
		$this->assertSame('test_true', $settings->getApiClientId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setPaymentContractId('test_true'));
		$this->assertSame('test_true', $settings->getPaymentContractId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setApiUsername('test_true'));
		$this->assertSame('test_true', $settings->getApiUsername());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setApiPassword('test_true'));
		$this->assertSame('test_true', $settings->getApiPassword());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setApiClientId('test_true'));
		$this->assertSame('test_true', $settings->getApiClientId());

		$this->assertInstanceOf(VerifoneHostedCartSettings::class, $settings->setEntityId('test_true'));
		$this->assertSame('test_true', $settings->getEntityId());

		$settings = new VerifoneHostedCartSettings();
		$this->assertNull($settings->getEntityId());
		$this->assertNull($settings->getApiPassword());
		$this->assertNull($settings->getApiClientId());
		$this->assertNull($settings->getRegion());
		$this->assertNull($settings->getApiUsername());
		$this->assertNull($settings->getCurlVerifySsl());
		$this->assertNull($settings->get3dsContractId());
		$this->assertNull($settings->getPaymentContractId());
	}

	public function test_Refund() {
		$orderMock = $this->getMockBuilder( Order::class )
		                     ->disableOriginalConstructor()
		                     ->getMock();

		$date = new \DateTime('now');
		$expected_arr = [
			'amount'    => '1000',
			'reason'    => 'test reason',
			'id'        => 'test id',
			'referenceId' => '',
			'createdDateTime'  => $date->format( 'Y-m-d\TH:i:s\Z' ),
		];
		$amount = $expected_arr['amount'];
		$reason = $expected_arr['reason'];
		$transactionId = $expected_arr['id' ];

		$refund_class = new VerifoneHostedRefund( $amount, $reason, $orderMock, $transactionId);

		$this->assertEquals( $refund_class->toJson() , json_encode( $expected_arr ) );
		$this->assertEquals( strlen( json_encode( $expected_arr ) ), strlen(  $refund_class->toJson() ) );
	}
}
